package com.docus.server.collect.web.service;

import com.docus.core.util.DateUtil;
import com.docus.core.util.json.JSON;
import com.docus.infrastructure.redis.service.IdService;
import com.docus.log.executor.TrackRetrySpringExecutor;
import com.docus.log.handler.IJobHandler;
import com.docus.log.handler.impl.MethodJobHandler;
import com.docus.server.archivefile.infrastructure.dao.ITaskMessageRetryLogDao;
import com.docus.server.entity.TaskMessage;
import com.docus.server.entity.TaskMessageRetryLog;
import com.docus.server.enums.StateEnum;
import com.docus.server.util.ExceptionUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.retry.annotation.Backoff;
import org.springframework.retry.annotation.Retryable;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;

/**
 * 重试service
 */
@Service
@Slf4j
public class TrackRetryService {
    @Resource
    private ITaskMessageRetryLogDao taskMessageRetryLogDao;
    @Resource
    private IdService idService;

    @Async("recordMessage")
    @Retryable(value = Exception.class, maxAttempts = 3, backoff = @Backoff(delay = 2000L, multiplier = 1.5))
    public void doRetry(TaskMessage message) throws Exception {

        TaskMessageRetryLog taskMessageRetryLog = beforeProcess(message);

        try {
            IJobHandler jobHandler = TrackRetrySpringExecutor.loadJobHandler(message.getRetryKey());
            String name = ((MethodJobHandler) jobHandler).getMethod().getParameterTypes()[0].getName();
            Object o = JSON.fromJSON(message.getJsonStr(), Class.forName(name));
            jobHandler.execute(o);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            afterThrowingProcess(ExceptionUtils.getExceptionMessage(e), taskMessageRetryLog);
        }

        taskMessageRetryLogDao.save(taskMessageRetryLog);
    }

    private void afterThrowingProcess(String errorMsg, TaskMessageRetryLog taskMessageRetryLog) {
        taskMessageRetryLog.setErrorMsg(errorMsg);
        taskMessageRetryLog.setState(StateEnum.FAIL);
    }

    private TaskMessageRetryLog beforeProcess(TaskMessage message) {
        TaskMessageRetryLog taskMessageRetryLog = new TaskMessageRetryLog();
        BeanUtils.copyProperties(message, taskMessageRetryLog);
        taskMessageRetryLog.setMessageId(message.getId());
        taskMessageRetryLog.setId(idService.getDateSeq());
        taskMessageRetryLog.setCreateTime(DateUtil.now());
        taskMessageRetryLog.setUpdateTime(DateUtil.now());
        return taskMessageRetryLog;
    }

}
